(vl-load-com)

;;;======================================================;;;
;;;====================-   ENTITE   -====================;;;
;;;======================================================;;;

(defun make_lwpline (LstDxf v /)
  (entmake
    (append
      (list
	'(0 . "LWPOLYLINE")
	'(100 . "AcDbEntity")
	'(67 . 0)
	'(410 . "Model")
	'(8 . "0")
	(cons '60 v)
	'(100 . "AcDbPolyline")
      )
      (list (cons '90 (length LstDxf)))
      (list (cons '43 0.0))
      (apply
	'append
	(mapcar
	  (function
	    (lambda (x) (list (cons '10 (v2d x)) (cons '42 0.0)))
	  )
	  LstDxf
	)
      )
    )
  )
  (entlast)
)

(defun make_line (LstDxf v /)
  (entmake
    (append
      '((0 . "LINE") (67 . 0) (410 . "Model") (8 . "0"))
      (list (cons '60 v))
      (mapcar (function (lambda (x y) (cons x (v2d y))))
	      '(10 11)
	      LstDxf
      )
    )
  )
  (entlast)
)

(defun make_arc	(LstDxf v /)
  (entmake
    (append
      '((0 . "ARC") (67 . 0) (410 . "Model") (8 . "0"))
      (list (cons '60 v))
      (mapcar (function	(lambda	(x y)
			  (cons	x
				(if (listp y)
				  (v2d y)
				  y
				)
			  )
			)
	      )
	      '(10 50 51 40)
	      LstDxf
      )
    )
  )
  (entlast)
)

(defun make_ray	(LstDxf v /)
  (entmake
    (append
      (list
	'(0 . "RAY")
	'(100 . "AcDbEntity")
	'(67 . 0)
	'(410 . "Model")
	'(8 . "0")
	(cons '60 v)
	'(100 . "AcDbRay")
      )
      (mapcar (function (lambda (x y) (cons x (v2d y))))
	      '(10 11)
	      LstDxf
      )
    )
  )
  (entlast)
)


(defun make_point (LstDxf v /)
  (entmake (append
	     '((0 . "POINT") (410 . "Model") (8 . "0"))
	     (list (cons '60 v))
	     (list (cons '10 (v2d LstDxf)))
	   )
  )
  (entlast)
)

;;;======================================================;;;
;;;====================- FIN ENTITE -====================;;;
;;;======================================================;;;

;;;======================================================;;;
;;;=======================- LISTE -======================;;;
;;;======================================================;;;

;; Manipulation des listes

;;; POSITION - retourne l'index du premier lment dans la liste,
;;; base 0, ou nil si non trouv
;;;   (position x '(a b c)) -&gt; nil, (position b '(a b c d)) -&gt; 1
(defun position	(x lst / ret)
  (if (not (zerop (setq ret (length (member x lst)))))
    (- (length lst) ret)
  )
)

;;; CONSP  - un lisp non vide
(defun consp (x) (and x (listp x)))

;; BUTLAST
;; Retourne la liste sans le dernier lment

(defun butlast (lst) (reverse (cdr (reverse lst))))

;; VAL_DXF
;; Valeur du code dxf d'une entit
(defun val_dxf (code ent) (cdr (assoc code (entget ent))))

;; DXF
;; Retourne la liste des donnes associ au code dxf
(defun dxf (code alst)
  (mapcar 'cdr (massoc code alst))
)

;; MASSOC
;; assoc multiple, retourne toutes les clef key
(defun massoc (key alst)
  (apply 'append
	 (mapcar '(lambda (x)
		    (if	(= (car x) key)
		      (list x)
		    )
		  )
		 alst
	 )
  )
)

;; MEMBER-NTH

(defun member-nth (expr lst n)
  (and (listp lst)
       (not (zerop n))
       (progn
	 (setq lst (append (list 'nil) lst))
	 (repeat n
	   (setq lst (member expr (cdr lst)))
	 )
	 t
       )
  )
  lst
)

;; REPEAT_N - Repete l'element ele n fois
;;; (repeat_n 'a 5) => '(a a a a a)
(defun repeat_n	(ele n / l)
  (repeat n
    (setq l (append (list ele) l))
  )
  l
)


;;; APPENDLST - Ajoute une liste dans une liste
;;; (appendlst '(1 2 3) '(4 5)) => '(1 2 3 (4 5))
(defun appendlst (l1 l2)
  (append l1 (list l2))
)

;;; CONSLST - Ajoute une liste dans une liste
;;; (appendlst '1 '(2 3)) => '(1 (2 3))
(defun conslst (l1 l2)
  (append (list l1) (list l2))
)

;;; list behind (including) the nth element
;;;   (std-nthcdr 1 '(0 1 2 3) => '(1 2 3)
(defun nthcdr (i lst)
  (repeat i
    (setq lst (cdr lst))
  )
  lst
)

;; CAR2LAST
;; Dplace le premier lment  la fin de la liste
(defun car2last	(lst)
  (reverse (cons (car lst) (reverse (cdr lst))))
)

;;; ROT1 - mettre le premier lment  la fin, version simple,
;;;        (rotation par un)
(defun rot1 (lst) (append (cdr lst) (list (car lst))))

(defun rot2 (lst) (rot1 (rot1 lst)))

;;; DIVISE_ELE
;;; Divise une liste en sous-liste par l'lment ele
(defun divise_ele (ele lst)
  (if (setq lst (remove-left ele lst))
    (cons (trunc ele lst)
	  (divise_ele ele (cdr (member ele lst)))
    )
  )
)

;;; CONCAT_ELE
;;; Concatene les sous-listes avec l'element ele
(defun concat_ele (sep lst)
  (if (and (listp lst) (cadr lst))
    (append (car lst) (cons sep (concat_ele sep (cdr lst))))
    (car lst)
  )
)

;;; REPLACE
;;; Remplace les lments OLD_ELE par NEW_ELE
(defun replace (old_ele new_ele lst)
  (if lst
    (mapcar '(lambda (x)
	       (if (= x old_ele)
		 new_ele
		 x
	       )
	     )
	    lst
    )
  )
)

;; REMOVE_DOUBLES
;; Supprime tous les doublons d'une liste

(defun remove_doubles (lst)
  (if lst
    (cons (car lst) (remove_doubles (vl-remove (car lst) lst)))
  )
)

;; REMOVE_DOUBLES_ELE
;; Supprime les doublons ELE d'une liste

(defun remove_doubles_ele (ele lst)
  (if (member ele lst)
    (append
      (trunc ele lst)
      (cons (car (member ele lst))
	    (remove_doubles_ele ele (remove-left ele (member ele lst)))
      )
    )
    lst
  )
)

;; REMOVE-I
;; Retourne la liste prive de l'lment  l'indice spcifi
;; (premier lment = 0)

(defun remove-i	(ind lst)
  (if (or (zerop ind) (null lst))
    (cdr lst)
    (cons (car lst) (remove-i (1- ind) (cdr lst)))
  )
)

;; REMOVE_FT
;; Retourne la liste prive des lements du premier indice
;; au second indice spcifi (premier lment = 0)

(defun remove_ft (lst from to)
  (cond	((or (null lst) (zerop to)) (cdr lst))
	((< 0 from)
	 (cons (car lst) (remove_ft (cdr lst) (1- from) (1- to)))
	)
	(T (remove_ft (cdr lst) (1- from) (1- to)))
  )
)

;; REMOVE-ELE
;; Retourne la liste sans la premire occurence de l'expression

(defun remove-ele (ele lst)
  (if (equal ele (car lst))
    (cdr lst)
    (cons (car lst) (remove-ele ele (cdr lst)))
  )
)

;;; REMOVE - Enlve un article d'une liste (les lments en doubles sont permis)
;;;   (remove 0 '(0 1 2 3 0)) -&gt; (1 2 3)
					; (c) par Serge Volkov

(defun remove (ele lst)
  (apply 'append (subst nil (list ele) (mapcar 'list lst)))
)

;;; REMOVE-IF - Enlvement conditionnel d'un article d'une liste simple
;;; fun demande exactement 1 argument
;;;   (remove-if 'zerop '(0 1 2 3 0)) -&gt; (1 2 3)
;;;   (remove-if 'numberp '(0 (0 1) "")) -&gt; ((0 1) "")

(defun remove-if (fun from)
  (cond	((atom from) from)		;nil or symbol (return that)
	((apply fun (list (car from))) (remove-if fun (cdr from)))
	(t (cons (car from) (remove-if fun (cdr from))))
  )
)

;;; ADD-IF-NOT - Ajout conditionnel d'un article d'une liste simple

(defun add-if-not (ele lst)
  (if (member ele lst) lst (append lst (list ele)))
)


;;; REMOVE-IF-NOT  - conserve tous les lments auxquels l'attribut s'applique
;;; disons : " conserve si ", doit tre dfini rcursivement, comme ceci
;;; par Vladimir Nesterowsky

(defun remove-if-not (pred lst)
  (apply 'append
	 (mapcar '(lambda (e)
		    (if	(pred e)
		      (list e)
		    )
		  )
		 lst
	 )
  )
)

;;; REMOVE-LEFT - enleve tous les lments ele en partant de la gauche
;;; tant que (car lst) = ele

(defun remove-left (ele lst)
  (if (and (listp lst) (equal (car lst) ele))
    (REMOVE-LEFT ele (cdr lst))
    lst
  )
)

;;; REMOVE-RIGHT - enleve tous les lments ele en partant de la droite
;;; tant que (car lst) = ele

(defun remove-right (ele lst)
  (if (listp lst)
    (reverse (REMOVE-LEFT ele (reverse lst)))
  )
)

;; SUBST-I
;; Remplace l'lment  l'indice spcifi par un nouvel lment

(defun subst-i (new ind lst)
  (if (or (zerop ind) (null lst))
    (cons new (cdr lst))
    (cons (car lst) (subst-i new (1- ind) (cdr lst)))
  )
)

;; Flatten List  -  Lee Mac
;; Transforms a nested list into a non-nested list

(defun flatten (l)
    (if (atom l)
        (list l)
        (append (flatten (car l)) (if (cdr l) (flatten (cdr l))))
    )
)

;; COMMON
;;; Retourne la liste des lments communs  lst1 et lst2

(defun common (l1 l2)
  (if l1
    (if	(member (car l1) l2)
      (cons (car l1) (common (cdr l1) l2))
      (common (cdr l1) l2)
    )
  )
)

;; EXCLUSIVE
;; Retourne une liste contenant les lments appartenant exclusivement  l1

(defun exclusive (l1 l2)
  (if l1
    (if	(member (car l1) l2)
      (exclusive (cdr l1) l2)
      (cons (car l1) (exclusive (cdr l1) l2))
    )
  )
)

;;; ADJOIN -  ajoute un lment dans une liste s'il n'y est pas dj 
;;; astuce : accepte des listes quotes aussi, tel que
;;;  (setq l '(1 2 3) (adjoin 0 'l)
;;;  -&gt; !l (0 1 2 3)
(defun adjoin (ele lst / tmp)
  (if (= (type lst) 'SYM)
    (setq tmp lst
	  lst (eval tmp)
    )
  )
  (setq	lst (cond ((member ele lst) lst)
		  (t (cons ele lst))
	    )
  )
  (if tmp
    (set tmp lst)
    lst
  )
)

;; EQUAL_EVERY
;; value si tous les membres d'une liste sont gaux  "fuzz" prs

(defun equal_every (lst fuzz)
  (vl-every (function (lambda (x) (equal x (car lst) fuzz)))
	    (cdr lst)
  )
)

;; TRUNC
;; Retourne la liste tronque  partir de la premire occurrence de l'expression
;; (liste complmentaire de celle retourne par MEMBER)

(defun trunc (expr lst)
  (if (and lst (not (equal (car lst) expr)))
    (cons (car lst) (trunc expr (cdr lst)))
  )
)

;; TRUNC-IF
;; Retourne la liste tronque  partir de la premire occurrence qui
;; retourne T  la fonction (complmentaire de celle retourne par VL-MEMBER-IF)

(defun trunc-if	(fun lst)
  (if (and lst (not ((eval fun) (car lst))))
    (cons (car lst) (trunc-if fun (cdr lst)))
  )
)


;; COMMON-FUZZ
;; Comme COMMON avec une tolrance dans la comparaison

(defun common-fuzz (l1 l2 fuzz)
  (if l1
    (if	(member-fuzz (car l1) l2 fuzz)
      (cons (car l1) (common-fuzz (cdr l1) l2 fuzz))
      (common-fuzz (cdr l1) l2 fuzz)
    )
  )
)

;; MEMBER-FUZZ
;; Comme MEMBER avec une tolrance dans la comparaison

(defun member-fuzz (expr lst fuzz)
  (while (and lst (not (equal (car lst) expr fuzz)))
    (setq lst (cdr lst))
  )
  lst
)

;; TRUNC-FUZZ 
;; Comme TRUNC avec une tolrance dans la comparaison

(defun trunc-fuzz (expr lst fuzz)
  (if (and lst (not (equal (car lst) expr)))
    (cons (car lst) (trunc-fuzz expr (cdr lst) fuzz))
  )
)

;;; SUBLIST
;;;  Retourne une sous-liste
;;;
;;; Arguments
;;; lst : une liste
;;; start : l'index de dpart de la sous liste (premier lment = 0)
;;; leng : la longueur (nombre d'lments) de la sous-liste (ou nil)
;;;
;;; Exemples :
;;; (sublist '(1 2 3 4 5 6) 2 2) -> (3 4)
;;; (sublist '(1 2 3 4 5 6) 2 nil) -> (3 4 5 6)

(defun sublist (lst start leng / n r)
  (if (or (not leng) (< (- (length lst) start) leng))
    (setq leng (- (length lst) start))
  )
  (setq n (+ start leng))
  (repeat leng (setq r (cons (nth (setq n (1- n)) lst) r)))
)

;; SPLIT-LIST
;;; Retourne une liste de sous-listes
;;;
;; Arguments
;; - lst : la liste  fractionner
;; - num : un entier, le nombre d'lments des sous listes
;; Exemples :
;; (split-list '(1 2 3 4 5 6 7 8) 2) -> ((1 2) (3 4) (5 6) (7 8))
;; (split-list '(1 2 3 4 5 6 7 8) 3) -> ((1 2 3) (4 5 6) (7 8))

(defun split-list (lst n)
  (if lst
    (cons (sublist lst 0 n) (split-list (sublist lst n nil) n))
  )
)

;; COMBI (gile)
;; Retourne la liste de toutes les combinaisons possibles
;; (combi '(1 2 3)) -> ((1) (1 2) (1 3) (1 2 3) (2) (2 3) (3))

(defun combi (lst / subr first tmp1 tmp2 tmp3 ret)
  (defun subr (l1 l2)
    (if	l2
      (cons (append l1 (list (car l2))) (subr l1 (cdr l2)))
    )
  )
  (while lst
    (setq first	(list (car lst))
	  tmp1	(subr first (cdr lst))
	  tmp2	(cons first tmp1)
    )
    (while tmp1
      (setq tmp3 (subr (car tmp1) (cdr (member (last (car tmp1)) lst)))
	    tmp1 (cdr (append tmp1 tmp3))
	    tmp2 (append tmp2 tmp3)
      )
    )
    (setq ret (append ret tmp2)
	  lst (cdr lst)
    )
  )
  ret
)

;; COMBI3 (gile)
;; Retourne la liste de toutes les combinaisons par 3
;; (combi3 '(1 2 3 4)) -> ((1 2 3) (1 2 4) (1 3 4) (2 3 4))

(defun combi3 (l / subr1 subr2)
  (defun subr1 (e1 e2 l)
    (if	l
      (cons (list e1 e2 (car l)) (subr1 e1 e2 (cdr l)))
    )
  )
  (defun subr2 (e l)
    (if	(cdr l)
      (append (subr1 e (car l) (cdr l)) (subr2 e (cdr l)))
    )
  )
  (if (cddr l)
    (append (subr2 (car l) (cdr l)) (combi3 (cdr l)))
  )
)

;; SORT (gile)
;; Trie une liste  l'aide d'une fonction de comparaison
;; fonctionne comme vl-sort sans supprimer les doublons
(defun sort (lst fun / merge tmp)
  (defun merge (l1 l2)
    (if	(and l1 l2)
      (if (fun (car l1) (car l2))
	(cons (car l1) (merge (cdr l1) l2))
	(cons (car l2) (merge l1 (cdr l2)))
      )
      (if l1
	l1
	l2
      )
    )
  )
  (setq	lst (mapcar 'list lst)
	fun (eval fun)
  )
  (while (cdr lst)
    (setq tmp lst
	  lst nil
    )
    (while (cdr tmp)
      (setq lst	(cons (merge (car tmp) (cadr tmp)) lst)
	    tmp	(cddr tmp)
      )
    )
    (and tmp (setq lst (cons (car tmp) lst)))
  )
  (car lst)
)

;; LST2STR
;; Concatne une liste et un sparateur en une chaine
;;
;; Arguments
;; lst : la liste  transformer en chaine
;; sep : le sparateur
;;
;; Exemples
;; (lst2str '(1 2 3) ",") -> "1,2,3"
;; (lst2str '("a" "b" "c") " ") -> "a b c"

(defun lst2str (lst sep)
  (if (cadr lst)
    (strcat (vl-princ-to-string (car lst))
	    sep
	    (lst2str (cdr lst) sep)
    )
    (vl-princ-to-string (car lst))
  )
)

;;;======================================================;;;
;;;=====================- FIN LISTE -====================;;;
;;;======================================================;;;

;;;======================================================;;;
;;;==================- VECTEUR&MATRICE -=================;;;
;;;======================================================;;;

;;; Quelques routines de calcul vectoriel et matriciel.

(defun vx (l) (car (append l '(0 0))))
(defun vy (l) (cadr (append l '(0 0))))
(defun vz (l) (caddr (append l '(0 0))))

;;=================== CALCUL VECTORIEL ===================;;

;; V+V
;; Retourne le somme de deux vecteur
;;
;; Arguments : deux vecteurs

(defun v+v (v1 v2) (mapcar '+ v1 v2))

;; V2D
;; Retourne un vecteur 2D
;;
;; Arguments : un vecteur
(defun v2d (l)
  (mapcar '+ (append l '(0.0 0.0)) '(0.0 0.0))
)

;; V3D
;; Retourne un vecteur 3D
;;
;; Arguments : un vecteur
(defun v3d (l)
  (mapcar '+ (append l '(0.0 0.0 0.0)) '(0.0 0.0 0.0))
)

;; VXS (gile)
;; Retourne le produit d'un vecteur par un scalaire
;;
;; Arguments : un vecteur et un rel

(defun vxs (v s) (mapcar (function (lambda (x) (* x s))) v))

;; VXV (gile)
;; Retourne le produit scalaire (rel) de deux vecteurs
;;
;; Arguments : deux vecteurs

(defun vxv (v1 v2) (apply '+ (mapcar '* v1 v2)))


;; V^V (gile)
;; Retourne le produit vectoriel (vecteur) de deux vecteurs
;;
;; Arguments : deux vecteurs

(defun v^v (v1 v2)
  (list	(- (* (cadr v1) (caddr v2)) (* (caddr v1) (cadr v2)))
	(- (* (caddr v1) (car v2)) (* (car v1) (caddr v2)))
	(- (* (car v1) (cadr v2)) (* (cadr v1) (car v2)))
  )
)

;; VUNIT (gile)
;; Retourne le vecteur unitaire d'un vecteur
;;
;; Argument : un vecteur

(defun vunit (v)
  ((lambda (l)
     (if (/= 0 l)
       (mapcar (function (lambda (x) (/ x l))) v)
     )
   )
    (distance '(0 0 0) v)
  )
)

;; VEC1 (gile)
;; Retourne le vecteur norm (1 unit) de sens p1 p2
;;
;; Arguments : deux points

(defun vec1 (p1 p2)
  ((lambda (d)
     (if (not (zerop d))
       (mapcar (function (lambda (x1 x2) (/ (- x2 x1) d))) p1 p2)
     )
   )
    (setq d (distance p1 p2))
  )
)

(defun colinear	(v1 v2 fuzz)
  (cond
    ((or (and (equal (car v1) 0 fuzz) (equal (car v2) 0 fuzz))
	 (and (equal (cadr v1) 0 fuzz) (equal (cadr v2) 0 fuzz))
     )
     t
    )
    ((or (equal (car v1) 0 fuzz)
	 (equal (car v2) 0 fuzz)
     )
     nil
    )
    (t
     (equal (/ (cadr v1) (car v1)) (/ (cadr v2) (car v2)) fuzz)
    )
  )
)

;; NORM_3PTS (gile)
;; Retourne le vecteur normal du plan dfini par 3 points
;;
;; Arguments : trois points

(defun norm_3pts (p0 p1 p2)
  (vunit (v^v (mapcar '- p1 p0) (mapcar '- p2 p0)))
)

;; Retourne la distance entre deux points
;;
;; Arguments : deux points

(defun dist (p1 p2)
  (sqrt
    (apply '+
	   (mapcar '(lambda (x) (expt x 2)) (mapcar '- p2 p1))
    )
  )
)

;; Retourne un vecteur
;;
;; Arguments : deux points

(defun vect (p1 p2) (mapcar '- p2 p1))

;; VROTATION
;;
;; Retourne le vecteur avec une rotation suivant l'angle en radian donn
;; Le vecteur doit tre en 2D
;; Arguments : un vecteur et un rel

(defun vrotation (v a)
  (cons	(- (* (car v) (cos a)) (* (cadr v) (sin a)))
	(cons (+ (* (car v) (sin a)) (* (cadr v) (cos a)))
	      (cddr v)
	)
  )
)

;; VNORME
;; Retourne la norme d'un vecteur
;;
;; Arguments : un vecteur

(defun vnorme (v1) (distance '(0 0 0) v1))

;; VANGLE
;; Retourne l'angle entre deux vecteurs
;;
;; Arguments : deux vecteurs

(defun vangle (v1 v2)
  (acos (/ (vxv v1 v2) (* (vnorme v1) (vnorme v2))))
)

;; VANGLE
;; Retourne l'angle d'un vecteur par rapport  l'horizontal
;;
;; Arguments : un vecteur

(defun vangleh (v)
  (angle '(0 0) v)
)

;; VPERP
;; Retourne le vecteur perpendiculaire  un vecteur
;;
;; Arguments : un vecteurs

(defun vperp (v)
  (cons (* (cadr v) -1.0) (cons (car v) (cddr v)))
)

;; Midpoint  -  Lee Mac
;; Returns the midpoint of two points

(defun mid (a b)
  (mapcar (function (lambda (a b) (/ (+ a b) 2.0))) a b)
)

;; Trigo  -  (gile)

(defun Trigo (p1 p2 p3)
  (< (sin (- (angle p1 p2) (angle p1 p3))) -1e-14)
)

;; Exemples d'utilisation -------------------------------

;; Project Point onto Line  -  Lee Mac
;; Projects pt onto the line defined by p1,p2

(defun ProjectPointToLine (pt p1 p2 / nm)
  (setq	nm (mapcar '- p2 p1)
	p1 (trans p1 0 nm)
	pt (trans pt 0 nm)
  )
  (trans (list (car p1) (cadr p1) (caddr pt)) nm 0)
)

;; LINEARP (gile)
;; Retourne T si tous les points de la liste sont aligns
;;
;; Arguments : une liste de points

(defun linearp (lst)
  (or
    (null (cddr lst))
    (and (or (equal (vec1 (car lst) (cadr lst))
		    (vec1 (car lst) (caddr lst))
		    1e-9
	     )
	     (equal (vec1 (car lst) (cadr lst))
		    (vec1 (caddr lst) (car lst))
		    1e-9
	     )
	 )
	 (linearp (cdr lst))
    )
  )
)

;; PARALLELP (gile)
;; Retourne T si les segments p1 p2 et p3 p4 sont parallles
;;
;; Arguments : quatre points

(defun parallel-p (p1 p2 p3 p4)
  (equal '(0 0 0)
	 (v^v (mapcar '- p1 p2) (mapcar '- p3 p4))
	 1e-9
  )
)

;; PERPENDICULARP (gile)
;; Retourne T si les segments p1 p2 et p3 p4 sont perpendiculaires
;;
;; Arguments : quatre points

(defun perpendicularp (p1 p2 p3 p4)
  (equal (vxv (mapcar '- p2 p1) (mapcar '- p4 p3)) 0 1e-9)
)

;; COPLANP
;; Retourne T si tous les points de la liste sont coplanaires
;;
;; Arguments : une liste de points

(defun coplanp (lst)
  (or
    (null (cdddr lst))
    (and
      (equal (vxv (v^v (vec1 (car lst) (cadr lst))
		       (vec1 (car lst) (caddr lst))
		  )
		  (vec1 (car lst) (cadddr lst))
	     )
	     0.0
	     1e-9
      )
      (coplanar-p (cdr lst))
    )
  )
)

;; ELEV (gile)
;;  Retourne l'lvation du point pt par rapport au plan dfini par un point 
;; du plan et sa normale
;;
;; Arguments
;; pt : le point dont on cherche l'lvation
;; org : un point queconque du plan de projection
;; nor : le vecteur normal du plan de projection

(defun elev (pt org nor)
  (/ (vxv nor (mapcar '- pt org)) (distance '(0 0 0) nor))
)

;; PROJ_PT (gile)
;; Retourne les coordonnes de la projection orthogonale
;; du point pt sur le plan dfini par son origine et sa normale
;;
;; Arguments
;; pt : le point dont on cherche la projection
;; org : un point queconque du plan de projection
;; nor : le vecteur normal du plan de projection

(defun proj_pt (pt org norm)
  (mapcar '-
	  pt
	  (mapcar
	    (function
	      (lambda (x)
		(* x
		   (cos (angle_3pts org (mapcar '+ org norm) pt))
		   (distance org pt)
		)
	      )
	    )
	    norm
	  )
  )
)

;; ILP (gile)
;;  Retourne le point d'intersection de la droite dfinie par p1 p2
;; et du plan dfini par un point et sa normale.
;;
;; Arguments
;; p1 et p2 : les points dfinissant la droite dont on cherche l'intersection
;; org : un point queconque du plan d'intersection
;; nor : le vecteur normal du paln d'intersection

(defun ilp (p1 p2 org nor / scl)
  (if (and
	(/= 0 (setq scl (vxv nor (mapcar '- p2 p1))))
	(setq scl (/ (vxv nor (mapcar '- p1 org)) scl))
      )
    (mapcar (function (lambda (x1 x2) (+ (* scl (- x1 x2)) x1)))
	    p1
	    p2
    )
  )
)


;;=================== CALCUL MATRICIEL ===================;;

;; M+M
;; Additionne 2 matrices
;;
;; Arguments : deux matrices

(defun m+m (m1 m2)
  (mapcar
    (function (lambda (v1 v2) (mapcar '+ v1 v2)))
    m1
    m2
  )
)

;; MXS
;; Multiplie une martice par un nombre
;;
;; Arguments : une matrice et un nombre

(defun mxs (m s)
  (mapcar (function (lambda (v) (vxs v s))) m)
)

;; TRP
;; transpose une matrice -Doug Wilson-
;;
;; Argument : une matrice

(defun trp (m) (apply 'mapcar (cons 'list m)))

;; MXV
;; Applique une matrice de transformation  un vecteur -Vladimir Nesterovsky-
;;
;; Arguments : une matrice et un vecteur

(defun mxv (m v)
  (mapcar (function (lambda (r) (vxv r v))) m)
)

;; MXM
;; Multiple (combine) deux matrices -Vladimir Nesterovsky-
;;
;; Arguments : deux matrices

(defun mxm (m q)
  (mapcar (function (lambda (r) (mxv (trp q) r))) m)
)

;; SQUARE-P
;; Evalue si une matrice est carre
;;
;; Argument : une matrice

(defun square-p	(m)
  (vl-every (function (lambda (v) (= (length v) (length m))))
	    m
  )
)

;; UNIFORM-P
;; Evalue si une matrice de transformation (3X3 ou 4X4) a un chelle uniforme

(defun uniform-p (m)
  (and (or (= 3 (length m))
	   (setq m (mapcar 'VMbutlast (VMbutlast m)))
       )
       (vl-every
	 (function
	   (lambda (v)
	     (equal (distance '(0 0 0) (car m))
		    (distance '(0 0 0) v)
		    1e-12
	     )
	   )
	 )
	 m
       )
  )
)

;; TMATRIX-VALID
;; Evalue si une matrice est valide pour une utilisation avec vlax-tmatrix

(defun tmatrix-valid (m)
  (and (= 4 (setq l (length m)))
       (square-p m)
       (uniform-p m)
  )
)

;; IMAT
;; Cre une matrice d'identit de dimension n
;;
;; Argument
;; n : la dimension de la matrice
;; i : toujours 0

(defun Imat (n i / sub)
  (defun sub (n i)
    (if	(< 0 n)
      (if (zerop i)
	(cons 1.0 (sub (1- n) n))
	(cons 0.0 (sub (1- n) (1- i)))
      )
    )
  )
  (if (< i n)
    (cons (sub n i) (imat n (1+ i)))
  )
)

;; GaussJordan (gile)
;; Applique la mthode d'limination de Gauss-Jordan  deux matrices
;;
;; Arguments : 2 matrices (la premire doit tre carre)

(defun GaussJordan (m1 m2 / mat len ind row piv new)
  (setq len (length m1))
  (if (= len (length m2))
    (progn
      (setq mat	(mapcar (function (lambda (x1 x2) (append x1 x2))) m1 m2)
	    ind	0
      )
      (while (< ind len)
	(setq row (nth ind mat))
	(if (zerop (car row))
	  (progn
	    (if	(vl-every 'zerop (mapcar 'car (member row mat)))
	      (setq mat nil)
	      (while (zerop (car row))
		(setq mat (append (vl-remove row mat) (list row))
		      row (nth ind mat)
		)
	      )
	    )
	  )
	)
	(if mat
	  (setq	piv (float (car row))
		new (vxs row (/ 1 piv))
		mat (mapcar
		      (function
			(lambda	(x)
			  (if (equal x row)
			    (cdr new)
			    (cdr
			      (mapcar
				'-
				x
				(vxs new (car x))
			      )
			    )
			  )
			)
		      )
		      mat
		    )
		ind (1+ ind)
	  )
	  (setq ind len)
	)
      )
      mat
    )
  )
)

;; INVERSE (gile)
;; Inverse une matrice carre (mthode Gauss-Jordan)
;;
;; Argument: la matrice
;; Retour : la matrice inverse ou nil (si non inversible)

(defun inverse (mat)
  (GaussJordan mat (Imat (length mat) 0))
)

;; COFACT (gile)
;; Retourne le cofacteur associ  l'lment ij d'une matrice
;;
;; Arguments
;; i = indice de la ligne (premire ligne = 1)
;; j = indice de la colonne (premire colonne = 1)
;; m = une matrice

(defun cofact (i j m)
  (* (determ
       (VMremove-i
	 (1- i)
	 (mapcar (function (lambda (x) (VMremove-i (1- j) x))) m)
       )
     )
     (expt -1 (+ i j))
  )
)

;; DETERM (gile)
;; Retourne le dterminant d'une matrice carr
;;
;; Argument : une matrice

(defun determ (m)
  (if (= 2 (length m))
    (- (* (caar m) (cadadr m)) (* (caadr m) (cadar m)))
    ((lambda (r n)
       (apply
	 '+
	 (mapcar
	   (function (lambda (x) (* x (cofact 1 (setq n (1+ n)) m))))
	   r
	 )
       )
     )
      (car m)
      0
    )
  )
)

;; ADJ-MAT (gile)
;; Retourne la matrice adjointe d'une matrice
;;
;; Argument : une matrice

(defun adj-mat (m / i)
  (setq i 0)
  (trp
    (mapcar
      (function
	(lambda	(v / j)
	  (setq	i (1+ i)
		j 0
	  )
	  (mapcar
	    (function (lambda (x) (cofact i (setq j (1+ j)) m)))
	    v
	  )
	)
      )
      m
    )
  )
)

;; INV-MAT (gile)
;; Retourne la matrice inverse d'une matrice (mthode des cofacteurs)
;;
;; Argument : une matrice

(defun inv-mat (m / d)
  (if (/= 0 (setq d (determ m)))
    (mxs (adj-mat m) (/ 1 d))
  )
)


;; Exemples d'applications ------------------------------

;; NENT2NENTP (gile)
;; Transforme la matrice 4X3 type nentsel en une matrice 4X4 type nentselp

(defun nent2nentp (mat)
  (append (trp mat) (list '(0.0 0.0 0.0 1.0)))
)

;; NENTP2NENT (gile)
;; Transforme la matrice 4X4 type nentselp en une matrice 4X3 type nentsel

(defun nentp2nentl (mat)
  (trp (reverse (cdr (reverse mat))))
)

;; RCS2WCS (gile)
;; Traduit les coordonnes du Systme de Coordonnes Reference (bloc ou xref) vers le SCG
;;
;; Arguments :
;; pt : un point dans le RCS
;; mat : une matrice de transformation, retourne par (caddr (nentsel)) ou (caddr (nentselp))

(defun RCS2WCS (pt mat)
  (setq pt (trans pt 0 0))
  (if (= 3 (length (car mat)))
    (mapcar '+ (mxv (trp (VMbutlast mat)) pt) (last mat))
    (mapcar '+
	    (mxv (mapcar 'VMbutlast (VMbutlast mat)) pt)
	    (VMbutlast (mapcar 'last mat))
    )
  )
)

;; WCS2RCS (gile) - d'aprs une ide de JoeBurke-
;; Traduit les coordonnes du SCG vers le Systme de Coordonnes Reference (bloc ou xref)
;;
;; Arguments :
;; pt : un point dans le SCG
;; mat : une matrice de transformation, retourne par (caddr (nentsel)) ou (caddr (nentselp))

(defun WCS2RCS (pt mat)
  (setq pt (trans pt 0 0))
  (if (= 3 (length (car mat)))
    (setq mat (append (trp mat) (list '(0.0 0.0 0.0 1.0))))
  )
  (setq mat (inv-mat mat))
  (mapcar '+ (mxv mat pt) (VMbutlast (mapcar 'last mat)))
)

;; TransNested (gile)
;; Convertit les coordonnes d'un point entre le SCG ou le SCU  et le SCR -systme de
;; coordones d'une rfrence (xref ou bloc) quelque soit son niveau d'imbrication-
;;
;; Arguments
;; pt : le point  convertir
;; rlst : la liste des entits "parents" de la plus imbriqu  celle insre dans
;;        l'espace courant -indentique  (last (nentsel)) ou (last (nentselp))
;; from to : comme avec trans : 0 pour le SCG, 1 pour le SCU courant, 2 pour le SCR

(defun TransNested (pt rlst from to / mat dep)
  (setq mat '((1 0 0) (0 1 0) (0 0 1)))
  (and (= 1 from) (setq pt (trans pt 1 0)))
  (and (= 2 to) (setq rlst (reverse rlst)))
  (and (or (= 2 from) (= 2 to))
       (while rlst
	 (setq geom (if	(= 2 to)
		      (RevRefGeom (car rlst))
		      (RefGeom (car rlst))
		    )
	       rlst (cdr rlst)
	       mat  (mxm (car geom) mat)
	       pt   (mapcar '+ (mxv (car geom) pt) (cadr geom))
	 )
       )
  )
  (if (= 1 to)
    (trans pt 0 1)
    pt
  )
)

;; RefGeom (gile)
;; Retourne une liste dont le premier lment est une matrice de transformation
;; (rotation, chelles, mormale) de dimension 3X3 et le second le point
;; d'insertion de l'objet dans son "parent" (xref, bloc ou espace)

(defun RefGeom (ename / elst ang norm)
  (setq	elst (entget ename)
	ang  (cdr (assoc 50 elst))
	norm (cdr (assoc 210 elst))
  )
  (list
    (mxm
      (mapcar (function (lambda (v) (trans v 0 norm T)))
	      '((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))
      )
      (mxm
	(list (list (cos ang) (- (sin ang)) 0.0)
	      (list (sin ang) (cos ang) 0.0)
	      '(0.0 0.0 1.0)
	)
	(list (list (cdr (assoc 41 elst)) 0.0 0.0)
	      (list 0.0 (cdr (assoc 42 elst)) 0.0)
	      (list 0.0 0.0 (cdr (assoc 43 elst)))
	)
      )
    )
    (trans (cdr (assoc 10 elst)) norm 0)
  )
)

;; RevRefGeom (gile)
;; Fonction inverse de RefGeom

(defun RevRefGeom (ename / entData ang norm mat)
  (setq	entData	(entget ename)
	ang	(- (cdr (assoc 50 entData)))
	norm	(cdr (assoc 210 entData))
  )
  (list
    (setq mat
	   (mxm
	     (list (list (/ 1 (cdr (assoc 41 entData))) 0.0 0.0)
		   (list 0.0 (/ 1 (cdr (assoc 42 entData))) 0.0)
		   (list 0.0 0.0 (/ 1 (cdr (assoc 43 entData))))
	     )
	     (mxm
	       (list (list (cos ang) (- (sin ang)) 0.0)
		     (list (sin ang) (cos ang) 0.0)
		     '(0.0 0.0 1.0)
	       )
	       (mapcar (function (lambda (v) (trans v norm 0 T)))
		       '((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))
	       )
	     )
	   )
    )
    (mapcar '-
	    (mxv mat (trans (cdr (assoc 10 entData)) norm 0))
    )
  )
)


;; Fonctions de manipulation des listes utilises dans les routines prcdentes

;; VMremove-i
;; Retourne la liste prive de l'lment  l'indice spcifi (premier lment = 0)
;;
;; Arguments : la liste et l'indice de l'lment  supprimer

(defun VMremove-i (ind lst)
  (if (or (zerop ind) (null lst))
    (cdr lst)
    (cons (car lst) (VMremove-i (1- ind) (cdr lst)))
  )
)

;; VMbutlast
;; Retourne la liste prive du dernier lment
;;
;; Argument : une liste

(defun VMbutlast (l) (reverse (cdr (reverse l))))

;;;======================================================;;;
;;;===============- FIN VECTEUR&MATRICE -================;;;
;;;======================================================;;;

;;;======================================================;;;
;;;=======================- COURBE -=====================;;;
;;;======================================================;;;

;;; Renvoi le cot sur lequel se situe un point
;;; dbut : T
;;; fin : nil

(defun Courbe-Position (Courbe Point)
  (if (> (Courbe-DistanceAuPoint
	   Courbe
	   (Courbe-PointLePlusProche Courbe Point)
	   't
	 )
	 (* (Courbe-Longueur Courbe) 0.5)
      )
    'nil
    't
  )
)


;;; Renvoi le point le plus proche
(defun Courbe-PointLePlusProche	(Courbe lPoint)
  (V2D (vlax-curve-getClosestPointTo
	 (vlax-ename->vla-object Courbe)
	 lPoint
       )
  )
)

;;; Renvoi la distance  un point donn  partir du debut ou de la fin
(defun Courbe-DistanceAuPoint (Courbe Point Fin / dist)
  (setq	dist (vlax-curve-getDistAtPoint
	       (vlax-ename->vla-object Courbe)
	       Point
	     )
  )
  (if Fin
    dist
    (- (Courbe-Longueur Courbe) dist)
  )
)

;;; Renvoi la liste des distances aux points donns  partir du debut ou de la fin
(defun Courbe-ListeDistancesAuxPoints (Courbe lPoint Fin / dist)
  (mapcar (function
	    (lambda (pt) (Courbe-DistanceAuPoint Courbe pt Fin))
	  )
	  lPoint
  )
)

;;; Renvoi le point  la distance donne  partir du debut ou de la fin
(defun Courbe-PointALaDistance (Courbe Dist Fin)
  (setq	Dist (if Fin
	       Dist
	       (- (Courbe-Longueur Courbe) Dist)
	     )
  )
  (V2D (vlax-curve-getPointAtDist
	 (vlax-ename->vla-object Courbe)
	 Dist
       )
  )
)

;;; Renvoi le point de depart et d'arriv
(defun Courbe-PtDepartPtArrive (Courbe)
  (list	(V2D
	  (vlax-curve-getStartPoint (vlax-ename->vla-object Courbe))
	)
	(V2D
	  (vlax-curve-getEndPoint (vlax-ename->vla-object Courbe))
	)
  )
)



;;; Renvoi la liste des points aux distances donnes  partir du debut ou de la fin
(defun Courbe-ListePointsAuxDistances (Courbe lDist Fin)
  (mapcar (function
	    (lambda (dist) (Courbe-PointALaDistance Courbe dist Fin))
	  )
	  lDist
  )
)

;;; Renvoi le parametre  la distance donne suivant  partir du debut ou de la fin
(defun Courbe-ParamALaDistance (Courbe Dist Fin)
  (setq	Dist (if Fin
	       Dist
	       (- (Courbe-Longueur Courbe) Dist)
	     )
  )
  (vlax-curve-getParamAtDist
    (vlax-ename->vla-object Courbe)
    Dist
  )
)

;;; Renvoi le parametre au point donne
(defun Courbe-ParamAuPoint (Courbe Point)
  (vlax-curve-getParamAtPoint
    (vlax-ename->vla-object Courbe)
    Point
  )
)


;;; Renvoi la derivee 1 d'une courbe
(defun Courbe-Derivee1 (Courbe Param /)
  (V2D (vlax-curve-getFirstDeriv
	 (vlax-ename->vla-object Courbe)
	 Param
       )
  )
)

;;; Renvoi la derivee 2 d'une courbe
(defun Courbe-Derivee2 (Courbe Param /)
  (V2D (vlax-curve-getSecondDeriv
	 (vlax-ename->vla-object Courbe)
	 Param
       )
  )
)

;;; Renvoi l'angle de la derivee 1 d'une courbe
(defun Courbe-AngleALaDistance (Courbe Dist Fin)
  (vangleh (Courbe-Derivee1
	     Courbe
	     (Courbe-ParamALaDistance Courbe Dist Fin)
	   )
  )
)

;;; Renvoi la longueur d'une courbe
(defun Courbe-Longueur (Courbe / prop vCourbe)
  (setq vCourbe (vlax-ename->vla-object Courbe))

  (- (vlax-curve-getDistAtParam
       vCourbe
       (vlax-curve-getEndParam vCourbe)
     )
     (vlax-curve-getDistAtParam
       vCourbe
       (vlax-curve-getStartParam vCourbe)
     )
  )

;;;  (cond
;;;    ((member '(0 . "ARC") (entget Courbe))
;;;     (setq prop 'ArcLength)
;;;    )
;;;    ((member '(0 . "CIRCLE") (entget Courbe))
;;;     (setq prop 'Circumference)
;;;    )
;;;    ((member '(0 . "LWPOLYLINE") (entget Courbe))
;;;     (setq prop 'Length)
;;;    )
;;;    ((member '(0 . "LINE") (entget Courbe))
;;;     (setq prop 'Length)
;;;    )
;;;  )
;;;  (if prop
;;;    (float
;;;      (vlax-get-property (vlax-ename->vla-object Courbe) prop)
;;;    )
;;;  )
)


;;; 0 : aucun prolongement
;;; 1 : prolonge la courbe 1
;;; 2 : prolonge la courbe 2
;;; 3 : prolonge les deux courbes

(defun Courbe-Intersection (Courbe1 Courbe2 Prolonge / point)
  (setq	point (vlax-variant-value
		(vla-intersectwith
		  (vlax-ename->vla-object Courbe1)
		  (vlax-ename->vla-object Courbe2)
		  Prolonge
		)
	      )
  )
  (if (> (vlax-safearray-get-u-bound point 1) 0)
    (mapcar 'V2D (split-list (vlax-safearray->list point) 3))
  )
)

(defun Courbe-Decaler (Courbe Dist Cote)
  (setq	Dist (*	Dist
		(if Cote
		  1
		  -1
		)
	     )
  )
  (vl-catch-all-apply
    'vla-Offset
    (list (vlax-ename->vla-object Courbe) Dist)
  )
  (entlast)
)

;; Bulge to Arc  -  Lee Mac
;; p1 - start vertex
;; p2 - end vertex
;; b  - bulge
;; dir  - trigo
;; Returns: (<center> <start angle> <end angle> <radius> <dir>)

(defun BulgeToArc (p1 b p2 / c r)
  (setq	r (/ (* (distance p1 p2) (1+ (* b b))) 4 b)
	c (polar p1 (+ (angle p1 p2) (- (/ pi 2) (* 2 (atan b)))) r)
  )
  (if (minusp b)
    (list c (angle c p2) (angle c p1) (abs r))
    (list c (angle c p1) (angle c p2) (abs r))
  )
)

;; Point to Bulge
;; c     - center
;; a1,a2 - start, end angle
;; r     - radius
;; Returns: <bulge>

(defun PtToBulge (c pt1 pt2 Dir)
  (setq	ang (if	Dir
	      (AngleTrigo (angle c pt1) (angle c pt2))
	      (AngleTrigo (angle c pt2) (angle c pt1))
	    )
  )
  (* (if Dir
       1
       -1
     )
     (tan (/ ang
	     4.0
	  )
     )
  )
)

;; Angle to Bulge
;; c     - center
;; a1,a2 - start, end angle
;; r     - radius
;; Returns: <bulge>

(defun AngleToBulge (c a1 a2 Dir)
  (setq	ang (if	Dir
	      (AngleTrigo a1 a2)
	      (AngleTrigo a2 a1)
	    )
  )
  (* (if Dir
       1
       -1
     )
     (tan (/ ang
	     4.0
	  )
     )
  )
)

;; Bulge Center  -  Lee Mac
;; p1 - start vertex
;; p2 - end vertex
;; b  - bulge
;; Returns the center of the arc described by the given bulge and vertices

(defun BulgeCenter (p1 b p2)
  (polar p1
	 (+ (angle p1 p2) (- (/ pi 2) (* 2 (atan b))))
	 (/ (* (distance p1 p2) (1+ (* b b))) 4 b)
  )
)

(defun AngleTrigo (AngleDep AngleArr)
  (ang<2pi (- AngleArr AngleDep))
)

;;;======================================================;;;
;;;====================- FIN COURBE -====================;;;
;;;======================================================;;;

;;;======================================================;;;
;;;=====================- SELECTION -====================;;;
;;;======================================================;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Selection -> liste
;;; Transforme un jeu de selection en liste de nom d'entit
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun SelEnList (sset / i bcl ss nb lst)
  (setq	i   -1
	lst '()
  )
  (defun bcl (ss nb)
    (if	(setq ent (ssname ss (setq nb (1+ nb))))
      (setq lst (cons ent (bcl ss nb)))
    )
  )
  (if sset
    (bcl sset i)
    'nil
  )
)

(defun SelEnLst	(ss)
  (if ss
    (vl-remove-if 'listp (mapcar 'cadr (ssnamex ss)))
  )
)

(defun ssmap (fct ss / n lst)
  (if (= 'PICKSET (type ss))
    (repeat (setq n (sslength ss))
      (setq
	lst (cons (apply fct (list (ssname ss (setq n (1- n))))) lst)
      )
    )
  )
)

;;;======================================================;;;
;;;==================- FIN SELECTION -===================;;;
;;;======================================================;;;

;;;======================================================;;;
;;;================- ESCALIER BALANCE -==================;;;
;;;======================================================;;;

;;; Variables globales pour la saisie
(setq Var_MarchePaliere	(if Var_MarchePaliere
			  Var_MarchePaliere
			  100.0
			)
      Var_NbGiron	(if Var_NbGiron
			  Var_NbGiron
			  1
			)
      Var_GironMax	(if Var_GironMax
			  Var_GironMax
			  280.0
			)
      Var_HtMarche	(if Var_HtMarche
			  Var_HtMarche
			  0.0
			)
      Var_SupprimerNdM	(if Var_SupprimerNdM
			  Var_SupprimerNdM
			  "Oui"
			)
      Var_SupprimerCrem	(if Var_SupprimerCrem
			  Var_SupprimerCrem
			  "Oui"
			)
)

(defun c:mdc (/)
  (princ "\n----- Modifier la crmaillre selon une courbe -----")
  (and
    (setq cremaillere (car (entsel "\nSelectionnez la cremaillre :")))
    (setq courbe (car (entsel "\nSelectionnez la nouvelle courbe de la crmaillre :")))
    (setq p		    (initget "Oui Non")
	  SupprimerCrem	    (getkword (strcat "\nVoulez-vous supprimer l'ancienne [Oui/Non] <"
					      Var_SupprimerCrem
					      "> : "
				      )
			    )
	  SupprimerCrem	    (if	SupprimerCrem
			      SupprimerCrem
			      Var_SupprimerCrem
			    )
	  Var_SupprimerCrem SupprimerCrem
    )
    (setq ;; On rcupre les points
	  liste_crem (dxf '10 (entget Cremaillere))
	  ;; si la crmaillre comprend une petite partie horizontale au dbut, on commence  l'inde 2
	  i	     (if (equal (caar liste_crem) (caadr liste_crem) 0.001)
		       1
		       2
		     )
    )
    (while (< i (length liste_crem))

      (setq ;; on recupre le nez de marche
	    pt	      (nth i liste_crem)
	    ;; on crer une ligne horizontale temporaire invisible
	    lignetemp (make_line (list pt (mapcar '+ pt '(1.0 0.0))) 1)
	    ;; Point d'intersection de cette ligne avec la courbe
	    nvpt      (car (Courbe-Intersection courbe lignetemp 2))
	    ;; on supprime la ligne
	    p	      (entdel lignetemp)
      )
      ;; Si il y a un point d'intersection, on remplace
      (if nvpt
	(setq ;; on remplace le point
	      liste_crem (subst-i nvpt i liste_crem)
	      ;; on modifie le point du dessous pour que les contremarches soient verticales
	      nvpt	 (list (car nvpt) (cadr (nth (1- i) liste_crem)))
	      ;; et on remplace
	      liste_crem (subst-i nvpt (1- i) liste_crem)
	)
      )
      (setq ;; on passe au nez de marche suivant
	    i
	     (+ i 2)
      )
    )
    ;; On cre la nouvelle crmaillre
    (make_lwpline liste_crem 0)
    ;; Et on supprime l'ancienne si c'est ok
    (if	(= SupprimerCrem "Oui")
      (entdel cremaillere)
      't
    )
  )

  (princ)
)

(defun c:tb (/)
  (princ "\n----- Traer les marches balances  partir de la crmaillre -----")

  (and
    (setq sel_ldf (entsel "\nSelectionnez la ligne de foule prs du dpart :"))
    (setq p		  (princ "\nSelectionnez les nez de marches :")
	  sel_nezdemarche (ssget '((-4 . "<OR")
				   (0 . "LINE")
				   (0 . "LWPOLYLINE")
				   (0 . "CIRCLE")
				   (0 . "ELLIPSE")
				   (0 . "RAY")
				   (0 . "XLINE")
				   (0 . "ARC")
				   (0 . "POLYLINE")
				   (0 . "SPLINE")
				   (-4 . "OR>")
				  )
			  )
    )
    (setq p		   (initget "Oui Non")
	  SupprimerNdM
			   (getkword (strcat "\nVoulez-vous supprimer les anciens nez de marche [Oui/Non] <"
					     Var_SupprimerNdM
					     "> : "
				     )
			   )
	  SupprimerNdM
			   (if SupprimerNdM
			     SupprimerNdM
			     Var_SupprimerNdM
			   )
	  Var_SupprimerNdM SupprimerNdM
    )
    (setq Cremaillere (car (entsel "\nSelectionnez la crmaillre :")))
    (setq sel_LimonC
	   (entsel
	     "\nSelectionnez le limon sur lequel se trouve la crmaillre prs du dpart :"
	   )
    )
    (setq LimonO (car (entsel "\nSelectionnez le limon oppos :")))

    (setq i	   0
	  liste_pt 'nil
	  ldf	   (car sel_ldf)
	  LimonC   (car sel_LimonC)
	  finldf   (Courbe-Position ldf (cadr sel_ldf))
	  fincrem  (Courbe-Position LimonC (cadr sel_LimonC))
	  p	   't
    )

;;; Liste des points d'intersection entre les nez de marche et la courbe
    (repeat (sslength sel_nezdemarche)
      (if (setq point (car (Courbe-Intersection ldf (ssname sel_nezdemarche i) 2)))
	(setq liste_pt (appendlst liste_pt point))
      )
      (setq i (1+ i))
    )

    (if	(= SupprimerNdM "Oui")
      (mapcar 'entdel (SelEnList sel_nezdemarche))
      't
    )

    (setq liste_dist   (Courbe-ListeDistancesAuxPoints ldf liste_pt finldf)
	  liste_dist   (vl-sort liste_dist '<)
	  liste_crem   (dxf '10 (entget Cremaillere))
	  liste_crem   (if (equal (caar liste_crem) (caadr liste_crem) 0.001)
			 (cons (car liste_crem) liste_crem)
			 liste_crem
		       )
	  liste_crem   (split-list liste_crem 2)
	  liste_crem   (mapcar 'car liste_crem)
	  pt_insertion (car liste_crem)
	  liste_crem   (mapcar (function (lambda (pt) (- (car pt) (car pt_insertion)))) (cdr liste_crem))
    )
    (mapcar (function (lambda (ptldf ptcrem)
			(setq lignetemp	(make_line (list ptldf ptcrem) 1)
			      ptint	(car (Courbe-Intersection LimonO lignetemp 2))
			)
			(entdel lignetemp)
			(make_line (list ptint ptcrem) 0)
		      )
	    )
	    (Courbe-ListePointsAuxDistances ldf liste_dist finldf)
	    (Courbe-ListePointsAuxDistances LimonC liste_crem fincrem)
    )
  )

  (princ)
)

(defun c:tm (/)
  (princ "\n----- Traer les marches rayonnantes  la ligne de foule -----")

  (and
    (setq sel_ldf (entsel "\nSelectionnez la ligne de foule prs du dpart :"))
    (setq LimonG (car (entsel "\nSelectionnez le limon gauche :")))
    (setq LimonD (car (entsel "\nSelectionnez le limon droit :")))
    (setq marche_paliere    (getdist (strcat "\nDimension de la marche palire <" (rtos Var_MarchePaliere 2) "> : "))
	  marche_paliere    (if	marche_paliere
			      marche_paliere
			      Var_MarchePaliere
			    )
	  Var_MarchePaliere marche_paliere
    )
    (setq p	   (initget 128 "Nb")
	  GironMax (getreal (strcat "\nIndiquez le giron maximum ou [Nb] <" (rtos Var_GironMax 2) ">: "))
	  nb_giron (cond
		     ((= GironMax "Nb")
		      (setq nb_giron	(getint (strcat "\nIndiquez le nb de giron <" (rtos Var_NbGiron 2) "> : "))
			    nb_giron	(if nb_giron
					  nb_giron
					  Var_NbGiron
					)
			    Var_NbGiron	nb_giron
		      )
		     )
		     ((numberp GironMax)
		      (setq Var_GironMax GironMax)
		      (1+ (fix (/ (Courbe-Longueur (car sel_ldf)) GironMax)))
		     )
		     (t Var_NbGiron)
		   )
    )
    (setq
      ldf	 (car sel_ldf)
      fin	 (Courbe-Position ldf (cadr sel_ldf))
      giron	 (/ (- (Courbe-Longueur ldf) marche_paliere) nb_giron)
      liste_dist '(0.0)
    )

    (princ (strcat "\nGiron : " (rtos giron 2)))

;;; Liste des positions des marches
    (repeat nb_giron
      (setq liste_dist (appendlst liste_dist (+ (last liste_dist) giron)))
    )

;;; On recupre la perpendiculaire  la courbe
    (setq liste_point
	   (mapcar
	     (function
	       (lambda (dist / pt)
		 (setq pt (Courbe-PointALaDistance ldf dist fin))
		 (list
		   pt
		   (V+V pt (VPERP (VXS (Courbe-Derivee1 ldf (Courbe-ParamALaDistance ldf dist fin)) 10)))
		 )
	       )
	     )
	     liste_dist
	   )

    )

;;; On trace les marches
    (foreach data liste_point
      (setq ligne (make_line data 1))
      (make_line
	(list (car (Courbe-Intersection LimonG ligne 2)) (car (Courbe-Intersection LimonD ligne 2)))
	0
      )
      (entdel ligne)
    )
  )

  (princ)
)

(defun c:tc (/)
  (princ "\n----- Traer la crmaillre -----")
  (and
    (setq sel_limon (entsel "\nSelectionnez le limon  dvelopper prs du dpart :"))
    (setq p		  (princ "\nSelectionnez les nez de marches :")
	  sel_nezdemarche (ssget '((-4 . "<OR")
				   (0 . "LINE")
				   (0 . "LWPOLYLINE")
				   (0 . "CIRCLE")
				   (0 . "ELLIPSE")
				   (0 . "RAY")
				   (0 . "XLINE")
				   (0 . "ARC")
				   (0 . "POLYLINE")
				   (0 . "SPLINE")
				   (-4 . "OR>")
				  )
			  )
    )
    (setq point_insertion (v2d (getpoint "\nIndiquez le point d'insertion du dvelopp :")))
    (setq ht_marche    (getdist (strcat "\nIndiquez la hauteur de marche (ou distance) <" (rtos Var_HtMarche 2) "> : "))
	  ht_marche    (if ht_marche
			 ht_marche
			 Var_HtMarche
		       )
	  Var_HtMarche ht_marche
	  ht_marche    (list 0.0 ht_marche)
    )
    (setq i	     0
	  liste_dist 'nil
	  limon	     (car sel_limon)
	  fin	     (Courbe-Position limon (cadr sel_limon))
	  p	     't
    )

;;; Liste des points d'intersection entre les nez de marche et la courbe
    (repeat (sslength sel_nezdemarche)
      (if (setq point (car (Courbe-Intersection limon (ssname sel_nezdemarche i) 2)))
	(setq liste_dist (appendlst liste_dist (Courbe-DistanceAuPoint limon point fin)))
      )
      (setq i (1+ i))
    )

;;; Tri des distances
;;; On initialise liste_point avec le point d'insertion et le premier decalage
;;; On y rajoute une premire ht
;;; Distance entre chaque point
;;; On crer les vecteurs

    (setq liste_dist  (vl-sort liste_dist '<)
	  liste_point (list point_insertion)
	  liste_point (if (equal (car liste_dist) 0.0 0.001)
			liste_point
			(appendlst liste_point (v+v point_insertion (list (car liste_dist) 0.0)))
		      )
	  liste_point (appendlst liste_point (v+v (last liste_point) ht_marche))
	  liste_diff  (mapcar '- (cdr liste_dist) liste_dist)
	  liste_diff  (mapcar (function (lambda (x) (list x 0.0))) liste_diff)

    )
    (foreach diff liste_diff
      (setq liste_point	(appendlst liste_point (v+v (last liste_point) diff))
	    liste_point	(appendlst liste_point (v+v (last liste_point) ht_marche))
      )
    )
    (setq derniere_dist	(last liste_dist)
	  lg_courbe	(Courbe-Longueur limon)
    )
    (if	(not (equal derniere_dist lg_courbe 0.001))
      (setq liste_point
	     (appendlst
	       liste_point
	       (mapcar '+ (last liste_point) (list (- lg_courbe derniere_dist) 0.0))
	     )
      )
    )
    (make_lwpline liste_point 0)
  )

  (princ)
)

(terpri)
(princ "\n========================================================================")
(princ "\n     Utilitaire d'aide au trac d'escalier  marches balances          ")
(princ "\n------------------------------------------------------------------------")
(princ "\n	- Traer les marches rayonnantes  la ligne de foule : TM")
(princ "\n	- Traer la crmaillre : TC")
(princ "\n	- Modifier la crmaillre selon une courbe : MDC")
(princ "\n	- Traer les marches balances  partir de la crmaillre : TB")
(princ "\n========================================================================")
(terpri)

(princ)
 ;|Visual LISP Format Options
(150 2 40 2 nil "Fin de " 100 9 0 0 0 T T nil T)
;*** NE PAS AJOUTER de texte au-dessous du commentaire! ***|;
